from decimal import Decimal
import re

__DIMEMSION_REGEX__ = re.compile(r'(?P<dim>\S)(?P<time>-?\d+)?')

__allowed_dimensions__ = ('L', 'M', 'T')


class Dimension:

    def __init__(self, dimension: str = ""):
        self.dim_dict = {
            "L": Decimal(0),
            "M": Decimal(0),
            "T": Decimal(0),
        }
        self.parse(dimension)

    @property
    def dimensionless(self):
        return self.__str__() == ""

    @property
    def L(self) -> Decimal:
        return self.dim_dict["L"]

    @property
    def M(self) -> Decimal:
        return self.dim_dict["M"]

    @property
    def T(self) -> Decimal:
        return self.dim_dict["T"]

    def to_str(self):
        output = ""
        for d in self.dim_dict:
            t = self.dim_dict[d]
            if t == 0:
                continue
            if t == 1:
                output += d
            else:
                output += f"{d}{t}"
        return output

    def parse(self, dimension: str) -> dict[str, Decimal]:
        result = __DIMEMSION_REGEX__.findall(dimension)
        for d, t in result:
            if d in __allowed_dimensions__:
                self.dim_dict[d] += Decimal(t or "1")
            else:
                raise ValueError(f"Dimension {d} is not allowed.")

    def __str__(self) -> str:
        return f"Dimension({self.to_str()})"

    def __repr__(self) -> str:
        return self.__str__()

    def __mul__(self, other: 'Dimension'):
        return Dimension(self.to_str() + other.to_str())

    def __truediv__(self, other: 'Dimension'):
        return Dimension(self.to_str() + (other ** -1).to_str())

    def __pow__(self, power: int):
        dim = Dimension(self.to_str())
        for d in dim.dim_dict:
            dim.dim_dict[d] *= power
        return dim

    def __eq__(self, __o: object) -> bool:
        if isinstance(__o, Dimension):
            for d in __allowed_dimensions__:
                if self.dim_dict[d] != __o.dim_dict[d]:
                    return False
            return True
        else:
            return False


L = Dimension('L')
M = Dimension('M')
T = Dimension('T')
DIMLESS = Dimension()



#!/usr/bin/env python

from setuptools import setup, find_packages, Command
import os
from os import path
import sys
from shutil import rmtree

# Package meta-data.
NAME = 'dimit'
DESC = 'Dimension of a quantity in Python language.'
URL = 'https://github.com/pboymt/dimit'
EMAIL = os.getenv('PypiEmail')
AUTHOR = os.getenv('PypiAuthor')
REQUIRES_PYTHON = '>=3.9.0'
VERSION: str = None  # '0.0.1'

# What packages are required for this module to be executed?
REQUIREMENTS = []

# What packages are optional?
EXTRAS = {
    # 'fancy feature': ['numpy'],
}

cwd = os.path.abspath(path.dirname(__file__))

try:
    with open(path.join(cwd, 'README.md'), encoding='utf-8') as f:
        LONG_DESC = f.read()
    with open(path.join(cwd, 'LICENSE'), encoding='utf-8') as f:
        LICENSE = f.read()
except FileNotFoundError:
    LONG_DESC = DESC
    LICENSE = 'GPLv3'

ABOUT = {}

if not VERSION:
    with open(path.join(cwd, NAME, '__version__.py')) as f:
        exec(f.read(), ABOUT)
else:
    ABOUT['__version__'] = VERSION


class TestCommand(Command):
    """
    Run tests
    """
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        os.system('pytest')


class UploadCommand(Command):
    """Support setup.py upload."""

    description = 'Build and publish the package.'
    user_options = []

    @staticmethod
    def status(s):
        """Prints things in bold."""
        print('\033[1m{0}\033[0m'.format(s))

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        try:
            self.status('Removing previous builds…')
            rmtree(os.path.join(cwd, 'dist'))
        except OSError:
            pass

        self.status('Building Source and Wheel (universal) distribution…')
        os.system(
            '{0} setup.py sdist bdist_wheel --universal'.format(sys.executable))

        self.status('Uploading the package to PyPi via Twine…')
        os.system('twine upload dist/*')

        self.status('Pushing git tags…')
        os.system('git tag v{0}'.format(ABOUT['__version__']))
        os.system('git push --tags')

        sys.exit()


setup(
    name=NAME,
    version=ABOUT['__version__'],
    author=AUTHOR,
    author_email=EMAIL,
    description=DESC,
    long_description=LONG_DESC,
    long_description_content_type='text/markdown',
    url=URL,
    license=LICENSE,
    packages=find_packages(exclude=('tests', 'docs')),
    requires=REQUIREMENTS,
    include_package_data=True,
    classifiers=[
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy'
    ],
    cmdclass={
        'upload': UploadCommand,
    }
)
